#include <cv.h>
#include <highgui.h>
#include <stdio.h>
#include <stdlib.h>
#include <opencv2/calib3d/calib3d.hpp>
#include "opencv2/core/core.hpp"
#include "opencv2/features2d/features2d.hpp"
#include "opencv2/highgui/highgui.hpp"

using namespace cv;

int num_boards = 34;
int frame_step = 5;
int board = 7;

int calibrate() {
   
 CvCapture* capture;
 
 int boardCorners  = board * board;
 CvSize board_sz = cvSize(board, board);
 
 namedWindow("Frame");
 namedWindow("Video");

 CvMat* image_points = cvCreateMat(num_boards * boardCorners,2,CV_32FC1);
 CvMat* object_points = cvCreateMat(num_boards * boardCorners,3,CV_32FC1);
 CvMat* point_counts = cvCreateMat(num_boards,1,CV_32SC1);
 CvMat* intrinsic_matrix = cvCreateMat(3,3,CV_32FC1);
 CvMat* distortion_coeffs = cvCreateMat(4,1,CV_32FC1);

 CvPoint2D32f* corners = new CvPoint2D32f[boardCorners];
 int corner_count;
 int successes = 0;
 int step, frame = 0;

 IplImage *image;
 char fileName[50];
 int imgNumb=0;
// for (int imgNumb = 0; imgNumb<=17;imgNumb++) 
// {
   sprintf(fileName, "calibration3/frame_%d.jpg", imgNumb);
   image = cvLoadImage(fileName);
   std::cout<<fileName<<"\n";
   
 IplImage *grayImg = cvCreateImage(cvGetSize(image),8,1);  

 while (imgNumb < 35){//successes < num_boards) {   
   sprintf(fileName, "calibration3/frame_%d.jpg", imgNumb);
   image = cvLoadImage(fileName);
   std::cout<<fileName<<"\n";
   imgNumb++;
   if (image == NULL)
       continue;
   int found = cvFindChessboardCorners(image, board_sz, corners, &corner_count,
                 CV_CALIB_CB_ADAPTIVE_THRESH | CV_CALIB_CB_FAST_CHECK | CV_CALIB_CB_NORMALIZE_IMAGE);

   cvCvtColor(image, grayImg, CV_BGR2GRAY);          
   cvFindCornerSubPix(grayImg, corners, corner_count, cvSize(11,11), cvSize(-1,-1),
                 cvTermCriteria(CV_TERMCRIT_EPS+CV_TERMCRIT_ITER, 30, 0.1 ));

   cvDrawChessboardCorners(image, board_sz, corners, corner_count, found);  
  
   // process correct board
   if (corner_count == boardCorners) {
     cvShowImage( "Frame_corners", image ); //show image - color
     //cvSaveImage("corners.jpg", image);
     step = successes * boardCorners;
     for( int i=step, j=0; j< boardCorners; ++i,++j ) {
       CV_MAT_ELEM(*image_points, float,i,0) = corners[j].x;
       CV_MAT_ELEM(*image_points, float,i,1) = corners[j].y;
       CV_MAT_ELEM(*object_points, float,i,0) = (float) j / board;
       CV_MAT_ELEM(*object_points, float,i,1) = (float) (j % board);
       CV_MAT_ELEM(*object_points, float,i,2) = 0.0f;
      }
     CV_MAT_ELEM(*point_counts, int, successes, 0) = boardCorners;   
     successes++;
     printf("\r%d successful frames out of %d collected.", successes, num_boards);
   }
   else
     cvShowImage("Frame", grayImg);//show image
     
  }
  //esc
  if (cvWaitKey(1500) == 27)
   return 0;

  //image = cvQueryFrame(capture); //get next image
 
 // cvShowImage("Video", image);
 //}
// cvDestroyWindow("Frame");

 //end getting frames -> calibrate
 
 CvMat* object_points2 = cvCreateMat(successes * boardCorners, 3, CV_32FC1);
 CvMat* image_points2 = cvCreateMat(successes * boardCorners, 2, CV_32FC1);
 CvMat* point_counts2 = cvCreateMat(successes, 1, CV_32SC1);

 for (int i = 0; i < successes * boardCorners; ++i) {
      CV_MAT_ELEM( *image_points2, float, i, 0) = CV_MAT_ELEM( *image_points, float, i, 0);
      CV_MAT_ELEM( *image_points2, float,i,1)   = CV_MAT_ELEM( *image_points, float, i, 1);
      CV_MAT_ELEM(*object_points2, float, i, 0) = CV_MAT_ELEM( *object_points, float, i, 0) ;
      CV_MAT_ELEM( *object_points2, float, i, 1)= CV_MAT_ELEM( *object_points, float, i, 1) ;
      CV_MAT_ELEM( *object_points2, float, i, 2)= CV_MAT_ELEM( *object_points, float, i, 2) ;
 }
 
 for (int i=0; i < successes; ++i) {
      CV_MAT_ELEM( *point_counts2, int, i, 0)= CV_MAT_ELEM( *point_counts, int, i, 0);  
 }

 // init intrinsic -> ratio 1.0

 CV_MAT_ELEM( *intrinsic_matrix, float, 0, 0 ) = 1.0f;
 CV_MAT_ELEM( *intrinsic_matrix, float, 1, 1 ) = 1.0f;
 //calibrate camera
 cvCalibrateCamera2(object_points2, image_points2, point_counts2, 
                      cvGetSize(image), intrinsic_matrix, distortion_coeffs,
                      NULL, NULL, CV_CALIB_FIX_ASPECT_RATIO);

 //save
 cvSave("Intrinsics.xml", intrinsic_matrix);
 cvSave("Distortion.xml", distortion_coeffs);
 printf("files saved \n");

 //correct display

 CvMat *intrinsic = (CvMat*)cvLoad("Intrinsics.xml");
 CvMat *distortion = (CvMat*)cvLoad("Distortion.xml");

 // build map

 IplImage* mapx = cvCreateImage(cvGetSize(image), IPL_DEPTH_32F, 1);
 IplImage* mapy = cvCreateImage(cvGetSize(image), IPL_DEPTH_32F, 1);
 cvInitUndistortMap(intrinsic,distortion,mapx,mapy);

 // show undistord and video

 cvNamedWindow("Undistort");
 imgNumb = 0;
 while (imgNumb<35) {
     sprintf(fileName, "calibration3/frame_%d.jpg", imgNumb);
   image = cvLoadImage(fileName);
   std::cout<<fileName<<"\n";
   imgNumb++;
   if (image == NULL)
       continue;
  IplImage *t = cvCloneImage(image);
  cvShowImage("Video", image);  // Show video
  //cvSaveImage("video.jpg", image);
  cvRemap( t, image, mapx, mapy);  // Undistort image
  cvReleaseImage(&t);
  cvShowImage("Undistort", image);  // Show corrected image
  //cvSaveImage("undistort.jpg", image);

  //esc
  if (cvWaitKey(1500) == 27)
   break;
 }
 return 0;
//}

}

