#include "Edge.h"
#include <math.h>
#include <map> 
#include "opencv2/core/core.hpp"


Edge::Edge(int x1, int x2, int y1, int y2) {
    this->init(x1, y1, x2, y2);
}
Edge::Edge(Point2f* a, Point2f* b) {
    this->init(a->x, a->y, b->x, b->y);
}
Edge::Edge(char* x1, char* y1, char* x2, char* y2) {
    this->init(atoi(x1), atoi(y1), atoi(x2), atoi(y2));    
}
Edge::Edge(const Edge& orig) {
}
Edge::~Edge() {
}
/**
 * initialize x1, y1, x2, y2 values and count edge length
 */
void Edge::init(int x1, int y1, int x2, int y2) {
    this->p1 = Point2f(x1, y1);
    this->p2 = Point2f(x2, y2);
    Point2f diff = Point2f(p2-p1);
    this->length = sqrt(pow(diff.x, 2) + pow(diff.y, 2));
    this->name = "";
}
/**
 * @param number
 * set door number to edge
 */
void Edge::addDoorNumber(string number) {
    this->name = number;
}
/**
 * @param image
 * @param scale
 * @param drawKeyPoints
 * draw keyPoint to map image
 */
void Edge::draw(Mat image, int scale, bool drawKeyPoints=false) {
  int thickness = 1;
  if (this->name != "") {
    thickness = 2;
  }
  int lineType = 8;
  line(image, Point2f(p1.x/scale, p1.y/scale), Point2f(p2.x/scale, p2.y/scale), Scalar( 0, 0, 0 ), thickness, lineType);
  if (drawKeyPoints) {
    for (std::map<double, Mat>::iterator iter = descriptors.begin(); iter!=descriptors.end(); ++iter) {
        Point2f temp = Point2f(p2-p1);
        Point2f p = Point2f(p1.x+(temp.x*iter->first), p1.y+(temp.y*iter->first));
        circle(image, Point2f(p.x/scale -1, p.y/scale -1), 2, Scalar(0, 0, 255), 1, 8);
    }
  }
}
/**
 * @return door number of edge
 */
string Edge::getDoorNumber() {
    std::cout << this->name;
  return name;
}
/**
 * @return length of edge
 */
double Edge::getLength() {
  return length;
}
/**
 * @return descriptors of edge - map<descrLengthFromStart, descrValue>
 */
std::map<double, Mat>* Edge::getDescriptors() {
    return &this->descriptors;
}
 
Mat* Edge::getDescriptorsValues() {
    Mat result = Mat::ones(0,64,CV_32F);    
    if (descriptors.size() == 0)
        return &result;
    std::cout<<"descriptors"<<descriptors.size()<<" \n";
    int i=0;
    for (std::map<double, Mat>::iterator iter = descriptors.begin(); iter!=descriptors.end(); ++iter) {
//        if (i==descriptors.size()-1)
//            continue;
        std::cout<<iter->second.rows<<"         i "<<i<<"\n";
        if (iter->second.rows>0) {
     //   std::cout<<iter->second<<" \n";
        result.push_back(iter->second);
        i++;
        }
    }
    std::cout<<result<<"   \n";
    std::cout<<"RETURNUJEM\n";
    return &result;
}
Point2f Edge::getDescriptor(Mat* descriptor) {
    Mat diff;
    for (std::map<double, Mat>::iterator iter = descriptors.begin(); iter!=descriptors.end(); ++iter) {
        cv::compare(iter->second, *descriptor, diff, cv::CMP_NE);
        if (cv::countNonZero(diff)==0) {
            if (p1.x == p2.x) {
                return Point2f(p1.x, p1.y+iter->first);
            } else if (p1.y == p2.y) {
                return Point2f(p1.x+iter->first, p1.y);
            }
        }
    }
    return Point2f(-1, -1);
}
/**
 * @param position
 * @param descriptor
 * add keyPoint with descriptor descriptor to position startPoint + position
 */
void Edge::addKeyPoint(double position, Mat descriptor) {
    this->descriptors.insert(std::make_pair(position, descriptor));
}

void Edge::removeKeyPoint(double len, Mat* descriptor) {
    this->descriptors.erase(len);
}
/**
 * @return center of edge
 */
Point2f Edge::getCenter() {
    return Point2f((p1.x+p2.x)/2, (p1.y+p2.y)/2);
}