#include "Map.h"
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <rapidxml.hpp>
#include "opencv2/highgui/highgui.hpp"
#include <strings.h>
#include <map>

#define SCALE 5

using namespace std;
using namespace rapidxml;
MapMap::iterator iter;

vector<DMatch> getMatches(Mat* train, Mat* query);

Map::Map() {
    this->width = 0;
    this->height = 0;
}

Map::Map(const Map& orig) {
}

Map::~Map() {
}
/**
 * @param fileName
 * read map from svg file
 */
void Map::readMap(char* fileName) {
  xml_document<> doc;
  xml_node<> *root_node;
  ifstream theFile(fileName);
  vector<char> buffer((istreambuf_iterator<char>(theFile)), istreambuf_iterator<char>());
  buffer.push_back('\0');
  doc.parse<0>(&buffer[0]);

  root_node = doc.first_node("svg");
  this->width = atoi(root_node->first_attribute("width")->value());
  this->height = atoi(root_node->first_attribute("height")->value());  
  for (xml_node<> *line_node = root_node->first_node("line"); line_node; line_node = line_node->next_sibling()) {
      Edge* newEdge = new Edge(line_node->first_attribute("x1")->value(), line_node->first_attribute("y1")->value(), 
            line_node->first_attribute("x2")->value(), line_node->first_attribute("y2")->value());
      xml_attribute<> *attr = line_node->first_attribute("doors");
      if (attr != 0) {
          newEdge->addDoorNumber(attr->value());
      }
      int id = atoi(line_node->first_attribute("lineId")->value());
      this->edges.insert(make_pair(id, newEdge));
  }
  this->descriptors = Mat(0,64,0);
//  this->gridMap = Mat::zeros(height/25 + 1, width/25 + 1, CV_8UC1);
//  for (iter = edges.begin(); iter!=edges.end(); ++iter) {
//    Edge* e = iter->second;
//    Point2f dir = Point2f(e->p2.x - e->p1.x, e->p2.y - e->p1.y);
//    if (dir.x == 0) {
//      if (fmod(e->p1.x, 25) != 0) {
//          for (int i=min(e->p1.y, e->p2.y)/25; i<=max(e->p1.y, e->p2.y)/25; i++) {
//              gridMap.at<int>(i, (int)e->p1.x/25) = 1;
//              //gridMap[i][e->p1.x/25] = 1;
//          }
//      }// else continue;
//    } else {
//      if (fmod(e->p1.y, 25) != 0) {
//          for (int i=min(e->p1.x, e->p2.x)/25; i<=max(e->p1.x, e->p2.x)/25; i++) {
//              cout << iter->first<<" i "<< (int)(e->p1.y/25) <<" j "<<i<<"\n";
//              gridMap.at<int>((int)e->p1.y/25, i) = 1;
//          }
//      }// else continue;
//    }
//  for (int i=0;i<gridMap.rows;i++) {
//    for (int j=0;i<gridMap.cols;j++) {
////      if (gridMap.at<int>(i, j) == 1) {
////      //  line(mapImage, Point2f(p1.x/scale, p1.y/scale), Point2f(p2.x/scale, p2.y/scale), Scalar( 0, 0, 0 ), 1, 8);
////      }
//    }
//  }
  //}
   
//  for (int i=0;i<gridMap.rows;i++) {
//    for (int j=0;i<gridMap.cols;j++) {
//      if (intersection(i, j)) {
//        gridMap.at<int>(i, j) = 1;
//      } else {
//        gridMap.at<int>(i, j) = 0;
//      }
//    }
//  }
 }

/**
 * draws map into mapImage
 */
  void Map::drawMap() {
    MapMap::iterator iter;
    Mat image(height/SCALE+10, width/SCALE+10, CV_8UC3, Scalar(255, 255, 255));    
    for (iter = edges.begin(); iter!=edges.end(); ++iter) {
      Edge* e = iter->second;
      e->draw(image, SCALE, false);
    }
    mapImage = image;
  }
  
/**
 * draws keyPoints in map into mapImage
 */
void Map::drawMapKeyPoints() {
  MapMap::iterator iter;
  for (iter = edges.begin(); iter!=edges.end(); ++iter) {
    Edge* e = iter->second;
    e->draw(mapImage, SCALE, true);
  }
}
/**
 * @return cross product of vector a and b
 */
float cross(Point2f* a, Point2f* b) {
  return a->x*b->y - a->y*b->x;
} 
/**
 * @param x1 first point of line1
 * @param y1 second point of line1
 * @param x2 first point of line2
 * @param y2 second point of line2
 * @return point of intersection of 2 lines || Point(-1, -1) if there is no intersection
 */
Point2f Map::intersection(Point2f* x1, Point2f* y1, Point2f* x2, Point2f* y2) {          
    Point2f x = *x2 - *x1;
    Point2f d1 = *y1 - *x1;
    Point2f d2 = *y2 - *x2;
    double cr = cross(&d1, &d2);
    double t = cross(&x, &d2)/cr;
    if ((cr == 0) || (t< 0) || (t > 1))
        return Point2f(-1, -1);
    Point2f result = *x1 + d1 * t;   
    if ((min(x2->x, y2->x) <= result.x) && (max(x2->y, y2->y) >= result.y) && 
        (max(x2->x, y2->x) >= result.x) && (min(x2->y, y2->y) <= result.y)){
        return result;
    }
    return Point2f(-1, -1);
}
/**
 * @return point of intersection of 2 edges || Point(-1, -1) if there is no intersection
 */
Point2f Map::intersection(Edge* e1, Edge* e2) {
    return intersection(&e1->p1, &e1->p2, &e2->p1, &e2->p2);
}
/**
 * @return distance of points p1 & p2
 */
float Map::countDist(Point2f p1, Point2f p2) {
    Point2f dif = Point2f(p2-p1);
    return sqrt(dif.dot(dif));
} 
/**
 * @param ray
 * @return pair<edgeId, pointOfIntersection> returns point of intersection of ray & nearest wall (edge from map)
 */
pair<int, Point2f> Map::getNearestIntersecton(Edge* ray) {
   int dist = max(width, height);
   Point2f interP = Point2f(-1,-1);
   int id;
   for (iter = edges.begin(); iter!=edges.end(); ++iter) {
      Point2f a = intersection(ray, iter->second);
      if (a.x != -1 && a.y != -1) {
          int newDist = countDist(a, ray->p1);
          if (newDist < dist) {
                id = iter->first;
                interP = a;
                dist = newDist;
          }
      }
   }
   return pair<int, Point2f>(id, interP);
}

pair<int, Point2f> Map::getKeyPointPosition(Mat* kpDescriptor) {
  pair<int, Point2f> result = pair<int, Point2f>(-1, Point2f());
  float dist = 4000;
  for (iter = edges.begin(); iter!=edges.end(); ++iter) {
      Mat* edgeDescriptors = iter->second->getDescriptorsValues();
      vector<DMatch> matches = getMatches(kpDescriptor, edgeDescriptors);
      if (matches.size() > 0) {
        for (int i = 0; i < matches.size(); i++) {
            if (matches[i].distance < dist) {
            Mat descr = kpDescriptor[matches[i].trainIdx];
            result = pair<int, Point2f>(iter->first, iter->second->getDescriptor(&descr));
            dist = matches[i].distance;
            }
        }
      }      
    }
  if (dist < 200) 
      return result ;
  else
  return pair<int, Point2f>(-1, Point2f());
}
  
void Map::updateKeyPointPosition(int edge, Point2f* position, Mat descriptor, Point2f robotPosition) {
    std::pair<int, Point2f> keyPointInMap = this->getKeyPointPosition(&descriptor);
    if (keyPointInMap.first != -1 && (Edge(&keyPointInMap.second, position).getLength() < 200)) {
        Point2f center = Edge(keyPointInMap.second.x, position->x, keyPointInMap.second.y, position->y).getCenter();
        Edge ray = Edge(&robotPosition, &center);
        pair<int, Point2f> newPositon = this->getNearestIntersecton(&ray);
        if (newPositon.second.x != -1 && newPositon.second.y != -1) {
           //vymaz stary
          this->removeKeyPoint(keyPointInMap.first, &keyPointInMap.second, &descriptor);
          this->addKeyPoint(newPositon.first, &newPositon.second, descriptor);
        }
    } else {
      this->addKeyPoint(edge, position, descriptor);
     // drawLine(&robotPosition, position);
    }
}

//  //distance, edge
//  MapMapPair Map::findInMap(cv::Mat* frame, int x, int y, cv::Point robotPosition) {          
//  }

/**
 * @return map width
 */
int Map::getWidth() {
   return width;
}
/**
 * @return map height
 */
int Map::getHeight() {
   return height;
}
/**
 * @return mapImage - updated using draw functions
 */
Mat Map::getImage() {
   return mapImage;
}
  Mat* Map::getDescriptors() {
      cout<<"tt"<<this->descriptors.type()<<"\n";
    return &this->descriptors;
}
/** 
 * draws line between p1 & p2 with color color
 */
void Map::drawLine(Point2f* p1, Point2f* p2, Scalar color) {
  line(mapImage, Point2f(p1->x/SCALE, p1->y/SCALE), Point2f(p2->x/SCALE, p2->y/SCALE), color, 1, 8);   
}
/** 
 * draws point p1 with radius r and color color
 */
void Map::drawPoint(Point2f p1, int r, Scalar color) {
  circle(mapImage, Point2f(p1.x/SCALE -r/2, p1.y/SCALE -r/2), r, color, 2, 8);
}

/** 
 * draws number s on position position
 */
void Map::drawNumber(string s, Point2f position) {
  putText(mapImage, s, Point2f(position.x/SCALE-20, position.y/SCALE), 
  CV_FONT_HERSHEY_COMPLEX_SMALL, 0.5, cvScalar(0,0,255), 1, CV_AA);
}

/**
 * @param edge - id of edge with keyPoint
 * @param position - position of keyPoint
 * @param descriptor - descriptor of keyPoint
 * add keyPoint to map
 */
void Map::addKeyPoint(int edge, Point2f* position, Mat descriptor) {
    double len = Edge(position, &this->edges.at(edge)->p1).getLength() / this->edges.at(edge)->getLength();
    //double len = Edge(&this->edges.at(edge)->p1, position).getLength();
    Point2f temp = Point2f(&this->edges.at(edge)->p2-&this->edges.at(edge)->p1);
    if (temp.x < 0 || temp.y <0)
        len *= -1;
    this->edges.at(edge)->addKeyPoint(len, descriptor);
    this->descriptors.push_back(descriptor);
}

/**
 * @param edge - edge id
 * @param position
 * @param descriptor
 * remove keyPoint from edge with id edge
 */
void Map::removeKeyPoint(int edge, Point2f* position, Mat* descriptor) {
    double len = Edge(position, &this->edges.at(edge)->p1).getLength() / this->edges.at(edge)->getLength();
    Point2f temp = Point2f(&this->edges.at(edge)->p2-&this->edges.at(edge)->p1);
    if (temp.x < 0 || temp.y <0)
        len *= -1;
    this->edges.at(edge)->removeKeyPoint(len, descriptor);
}

/**
 * @param matrix
 * export matrix matrix to string
 */
char* getMatToString(Mat* matrix) {
   static char s[500];
   char s2[50];
   s[0]=0;
   for (int i=0; i<matrix->cols; i++){
       double elem = matrix->at<double>(0, i);
       sprintf(s2, "%e ", elem);
       strncpy(s + strlen(s), s2, strlen(s2) + 1);
   }
   return s;
}

/**
 * @param fileName
 * save map descriptors to file fileName
 */
void Map::saveMap(char* fileName) {
  FILE *f;
  f = fopen(fileName, "w+");
  for (iter = this->edges.begin(); iter!=this->edges.end(); ++iter) {
      Edge* e = iter->second;
      if (e->getDescriptors() != NULL) {
        for (map<double, Mat>::iterator iter2 = e->getDescriptors()->begin(); iter2 != e->getDescriptors()->end(); ++iter2) {
          fprintf(f, "%d %f\n", iter->first, iter2->first);
          fprintf(f, "%s\n", getMatToString(&iter2->second));
        }
      }
  }
  fprintf(f, "%d\n", -1);
  fclose(f);
}
/**
 * @param fileName
 * read map descriptors from file fileName
 */
void Map::readMapDescriptors(char* fileName) {  
  FILE *f;
  f = fopen(fileName, "r");
  int edgeId;
  float length;
  while (fscanf(f, "%d", &edgeId)!=EOF){
    if (edgeId < 0)
       break;
    fscanf(f, "%f", &length);
    Mat descriptors = Mat::ones(1, 64, CV_32F);
    for (int i=0; i<64; i++) {
       float val;  
       fscanf(f, "%e", &val);   
       descriptors.at<float>(0, i) = (float)val;
    }  
    this->edges.at(edgeId)->addKeyPoint((double)length, descriptors);
    this->descriptors.push_back(descriptors);
  }
  fclose(f);
}
/**
 * @param fileName
 * read log with fileName from robot and count odometry error
 */
void Map::readLog(char* fileName) {
  FILE *f;
  f = fopen(fileName, "r");
  char line[256];
  char *pos;
  int lOtack, pOtack;
  float kompas;
  int startX, startY, endX, endY;
  Point2f startPosition, endPosition;
  std::vector< pair<int, int> > odometria;
  char frames[1000][20];
  fgets(line, 255, f);
  sscanf(line, "%d %d", &startX, &startY);
  startPosition = Point2f(startX, startY);
  fgets(line, 255, f);
  sscanf(line, "%d %d", &endX, &endY);
  endPosition = Point2f(endX, endY);
  fgets(line, 255, f); //title log line
  int i=0;
  while (fgets(line, 255, f) != NULL) {
      pos = index(line, ' ');
      *pos = 0;
      strncpy(frames[i], line, 19);
      i++;
      pos++;
      //frames.push_back(&line);
      sscanf(pos,"%*d %d %d %*d %*d %*d %*d %*d %*d %*d %*d %*d %f", &lOtack, &pOtack, &kompas);
      odometria.push_back(pair<int, int>(lOtack, pOtack));
      cout << line <<" "<< lOtack <<" "<< pOtack <<" "<< kompas <<"\n";
  }
  int start = round((odometria.at(0).first + odometria.at(0).second)/2);
  int end = round((odometria.at(odometria.size()-1).first + odometria.at(odometria.size()-1).second)/2);
  float scale = Edge(&endPosition, &startPosition).getLength() / (end - start);
  for (int i = 0; i<odometria.size();i++) {
      float posun = scale * (round((odometria.at(i).first + odometria.at(i).second)/2)-start);
      this->drawPoint(Point2f(startPosition.x, startPosition.y+posun), 2,  Scalar(255,0, 0));
    //  findInMap(frames[i], Point(startPosition.x, startPosition.y+posun), 0);
  }
  fclose(f);
}
/**
 * @return edges of map - map<edgeId, edgeObject>
 */
map<int, Edge*> Map::getEdges() {
    return edges;
}