#ifndef SURFNAV_H
#define	SURFNAV_H

#include <stdio.h>
#include <iostream>
#include <opencv2/calib3d/calib3d.hpp>
#include "opencv2/core/core.hpp"
#include "opencv2/features2d/features2d.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/nonfree/features2d.hpp"
#include <math.h>
#include <map>
#include "Map.h"

typedef pair<Point2f, Point2f> PairOfPoints;

using namespace cv;
  Mat descriptors1, descriptors2, descriptors, newDescriptors;
  vector<KeyPoint> keypoints1, keypoints2, keypoints, newKeypoints;
  //vector<Point*> positions;
  FlannBasedMatcher matcher;
  Map myMap;  
  IplImage* previousImage = NULL;
  float focalLength = 1620;//1.63277771e+03;
  CvMat *intrinsic, *distortion;
  IplImage* mapx;IplImage* mapy;
  Point2f principalPoint = Point2f(4.18555664e+02, 1.63277771e+03);//Point2f(360, 288);

  static int minHessian = 400;  
  SurfFeatureDetector detector(minHessian);  
  SurfDescriptorExtractor extractor;
  std::vector< DMatch > good_matches;

void init(IplImage* image) {
 intrinsic = (CvMat*)cvLoad("Intrinsics.xml");
 distortion = (CvMat*)cvLoad("Distortion.xml");
 mapx = cvCreateImage(cvSize(image->width,image->height), IPL_DEPTH_32F, 1);
 mapy = cvCreateImage(cvSize(image->width,image->height), IPL_DEPTH_32F, 1);
 cvInitUndistortMap(intrinsic,distortion,mapx,mapy);
}

/**
 * read and draw map from svg file
 * read image for drawing keypoints
 */
void init() {
  myMap = Map();
  myMap.readMap("plan.svg");
  myMap.drawMap();
  IplImage *img_2 = cvLoadImage("rucneMapovanie/frame_1.jpg", CV_LOAD_IMAGE_GRAYSCALE);
  init(img_2);
}

/**
 * show images with matched keypoints (contained on both images)
 */
void showResult(Mat* img_1, std::vector<KeyPoint>& keypoints_1, 
        Mat* img_2, std::vector<KeyPoint>& keypoints_2, 
        std::vector< DMatch > matches) {
  Mat img_matches, img_keypoints_1;
  drawMatches(*img_1, keypoints_1, *img_2, keypoints_2,
               matches, img_matches, Scalar::all(-1), Scalar::all(-1), 
               std::vector<char>(), DrawMatchesFlags::NOT_DRAW_SINGLE_POINTS);
  for (int i=0; i< good_matches.size(); i++) {
        keypoints.push_back(keypoints1[good_matches[i].trainIdx]);
        
  std::stringstream ss;
  ss << good_matches[i].trainIdx;   
  putText(img_matches, ss.str(), cvPoint(keypoints2[good_matches[i].queryIdx].pt.x, keypoints2[good_matches[i].queryIdx].pt.y), 
    FONT_HERSHEY_COMPLEX_SMALL, 0.8, cvScalar(255,0,0), 1, CV_AA);
  }
}

/**
 * return set of detected keypoints from img
 */
vector<KeyPoint> getKeypoints(Mat* img) {
  std::vector<KeyPoint> keypoints;
  detector.detect(*img, keypoints);
  return keypoints;
}

/**
 * get descriptors for set of keypoints from image img
 */
Mat getDescriptors(Mat* img, vector<KeyPoint> keypoints) {
    Mat descriptors;
    extractor.compute(*img, keypoints, descriptors);
    return descriptors;
}

/**
 * @param train - training set
 * @param query - query set
 * return set of matches which distance is lower than 100
 */
vector<DMatch> getMatches(Mat* train, Mat* query) {
  vector<DMatch> matches, robustMatch;  
  //query train
  matcher.match(*query, *train, matches);
  double max_dist = 0; double min_dist = 100;  
  for (int i = 0; i < matches.size(); i++) { 
    double dist = matches.at(i).distance;
    if (dist < min_dist) 
        min_dist = dist;
    if (dist > max_dist) 
        max_dist = dist;
  }
  for (int i = 0; i < matches.size(); i++ ) { 
    if (matches.at(i).distance < 2*min_dist) { 
      robustMatch.push_back(matches[i]); 
    }
  }
  return robustMatch;
}
/**
 * @param img1
 * @param img2
 * match images img1 & img2, initialize keypoints and descriptors values and count matches into good_matches
 */
void matchImages(Mat* img1, Mat* img2) {
  keypoints1 = getKeypoints(img1);
  keypoints2 = getKeypoints(img2);

  descriptors1 = getDescriptors(img1, keypoints1);
  descriptors2 = getDescriptors(img2, keypoints2);
  
  good_matches = getMatches(&descriptors1, &descriptors2);
}
/**
 * @param img
 * match image img with last image from sequence - previousImage
 */
void match(Mat* img) {
  newKeypoints = getKeypoints(img);
  newDescriptors = getDescriptors(img, newKeypoints);
  
  Mat mm = Mat(previousImage, false);
  matchImages(img, &mm);
  //if (myMap.getDescriptors().rows > 0) {
  //  good_matches = getMatches(myMap.getDescriptors(), newDescriptors);
  //}
  //drawKeypoints(img, good_matches, keypoints, Scalar::all(-1), DrawMatchesFlags::DEFAULT);
}

/*
 * match image with data in map and returns pair<keyPointKoordinates, robotPosition> 
 */
vector<PairOfPoints> matchWithMap(Mat* img) {
  vector<PairOfPoints> positions;//obr real
  imshow("test", *img);
  waitKey(1000);
  newKeypoints = getKeypoints(img);  
  if (newKeypoints.size()==0)
      return positions;
  newDescriptors = getDescriptors(img, newKeypoints);
  vector<DMatch> matches = getMatches(myMap.getDescriptors(), &newDescriptors);
  for (int ii=0;ii<myMap.getEdges().size();ii++) {
          if (ii==(myMap.getEdges().size()-1)) {
            return positions;
        }
  //for (map<int, Edge*>::iterator iter = myMap.getEdges().begin(); iter != myMap.getEdges().end(); ++iter) {      
    Mat* edgeDescriptors = myMap.getEdges().at(ii)->getDescriptorsValues();
            //iter->second->getDescriptorsValues();
    if (edgeDescriptors !=NULL)
    std::cout << " ----   " <<  edgeDescriptors->rows<<"    " <<  ii <<"\n";
    if (edgeDescriptors !=NULL)
    if (edgeDescriptors->rows > 0) {   
        vector<DMatch> matches = getMatches(edgeDescriptors, &newDescriptors);
    for (int i = 0; i < matches.size(); i++) {
        Mat aa = edgeDescriptors->row(matches[i].queryIdx);
        Point2f positionInMap = myMap.getEdges().at(ii)->getDescriptor(&aa);//iter->second
        pair<Point2f, Point2f> pp = make_pair(Point2f(newKeypoints[matches[i].queryIdx].pt), positionInMap);
        positions.push_back(pp);
    }        
    }
  }  
  return positions;
}
/**
 * match 2 images
 */
void surf() {
  Mat img_1 = imread("map/i19a.png", CV_LOAD_IMAGE_GRAYSCALE);
  Mat img_2 = imread("map/i19c.png", CV_LOAD_IMAGE_GRAYSCALE);
  matchImages(&img_1, &img_2);
  }

/**
 * @param focal - focal length of camera
 * @param pp - principal point of camera
 * @param kp - detected keyPoint
 * @return angle between principalPoint, camera and detected keyPoint
 */
double getAngle(float focal, Point2f pp, Point2f kp) {
    double tg_beta = (kp.x - pp.x) / focal;
    double angle = atan(tg_beta);
    return angle;
}
/**
 * @param focal - focal length of camera
 * @param pp - principal point
 * @param kp1 - first detected keyPoint
 * @param kp2 - second detected keyPoint
 * @return angle between 2 points and camera
 */
double getAngle(float focal, Point2f pp, Point2f kp1, Point2f kp2) {
    double angle1 = getAngle(focal, pp, kp1);
    double angle2 = getAngle(focal, pp, kp2);
    if ((pp.x > kp1.x && pp.y > kp1.y) || (pp.x < kp1.x && pp.y < kp1.y))
        return abs(angle1-angle2);
    else
        return angle1+angle2;
}
/**
 * @param deg
 * @return return radian value of angle deg
 */
float degToRad(float deg) {
    return ((deg * M_PI) / 180);
}
/**
 * @param p2
 * @param p1
 * @param orientation - rotation angle in rad
 * @return rotate point p2 around point p1 with angle orientation
 */
Point2f rotateZ(Point2f p2, Point2f p1, double orientation) {
  Point2f pp = Point2f(p2-p1);
  double x = pp.x * cos(orientation) - pp.y * sin(orientation);
  double y = pp.x * sin(orientation) + pp.y * cos(orientation);
  return Point2f(x+p1.x,y+p1.y);
}
/**
 * @param position - robot (camera) position
 * @param orientation - orientation of robot on map
 * @return line representing viewing ray
 */
Edge createRay(Point2f position, float orientation) {   
    int length = max(myMap.getWidth(), myMap.getHeight());
    Point2f p2 = Point2f(position.x+length, position.y);
    Point2f rotated = rotateZ(p2, position, orientation);
    return Edge(&position, &rotated);
}
/**
 * @param s1 - first center
 * @param r1 - first radius
 * @param s2 - second center
 * @param r2 - second radius
 * @return pair<intersectionPoint1, intersectionPoint2> of two circles
 */
pair<Point2f, Point2f> circleIntersection(Point2f s1, float r1, Point2f s2, float r2) {
  float d, a, h;
  d = sqrt((s1.x - s2.x)*(s1.x - s2.x) + (s1.y - s2.y)*(s1.y - s2.y));
  a = (r1*r1 - r2*r2 + d*d)/(2*d);
  h = sqrt(r1*r1 - a*a);
  Point2f p = Point2f(s2-s1);
  p.x = p.x*(a/d) + s1.x;
  p.y = p.y*(a/d) + s1.y;  
  float x3, y3, x4, y4;
  x3 = p.x + h*(s2.y - s1.y)/d;
  y3 = p.y - h*(s2.x - s1.x)/d;
  x4 = p.x - h*(s2.y - s1.y)/d;
  y4 = p.y + h*(s2.x - s1.x)/d;
  return pair<Point2f, Point2f>(Point2f(x3, y3), Point2f(x4, y4));
}

pair<Point2f, float> computeCenter(Point2f* x1, Point2f* x2, float rad) {
    if (x1->x > x2->x)
        return computeCenter(x2, x1, rad);
    float rad2 = rad/2;
    float len = Edge(x1, x2).getLength();
   
    float a1 = abs(len/2 / tan(rad2));
    float a2 = abs(len/2 / tan(M_PI/2-rad2));
    float r = (a1+a2)/2;
    float s = a1-r;
    Point2f v = Point2f((x2->y-x1->y)/len*s, (-1)*(x2->x-x1->x)/len*s);
    Point2f S = Edge(x1, x2).getCenter() + v;
    return pair<Point2f, float> (S, r);
}

Point2f findPartialPosition(Point2f* k1, Point2f* k2, float angle) {
   pair<Point2f, int> s = computeCenter(k1, k2, angle);
   return s.first;
}

/* 
 * @param frameImage
 * @param robotPosition
 * @param robotOrientation
 * @return detect and update keypoints position in map
 */
pair<double, int> findInMap(const IplImage* frameImage, Point2f robotPosition, float robotOrientation) {
  //initialize: newKeypoints, newDescriptors
  IplImage *t = cvCloneImage(frameImage);
  cvRemap(frameImage, t, mapx, mapy);  // Undistort image  
  if (previousImage == NULL) {
    previousImage = cvCloneImage(frameImage);
    return pair<double, int> (-1,-1);
  }
  Mat frame =Mat(frameImage, false);
  match(&frame);  
  
  Mat ress;
  //show view angle
  cvtColor(frame, ress, CV_GRAY2RGB);
  circle(ress, principalPoint, 3, Scalar(0,0,255), 3, 8);
  double a = getAngle(focalLength, principalPoint, Point2f(0.0, 0.0));
  Edge ray = createRay(robotPosition, robotOrientation+a); 
  std::pair<int, Point2f> temp = myMap.getNearestIntersecton(&ray);  
  myMap.drawLine(&robotPosition, &temp.second, Scalar(0, 0, 255));
  a = getAngle(focalLength, principalPoint, Point2f(ress.cols, ress.rows));
  ray = createRay(robotPosition, robotOrientation+a); 
  temp = myMap.getNearestIntersecton(&ray);  
  myMap.drawLine(&robotPosition, &temp.second, Scalar(0, 0, 255));   
     
  for (int i=0; i< good_matches.size(); i++) {
      //cout<< good_matches[i].queryIdx <<" "<< newDescriptors.rows<<"\n";
      if (good_matches[i].queryIdx >= newDescriptors.rows)
          continue;
     Mat descr = newDescriptors.row(good_matches[i].queryIdx);
     Point2f key_point = Point2f(newKeypoints[good_matches[i].queryIdx].pt.x, newKeypoints[good_matches[i].queryIdx].pt.y);

     double angle = getAngle(focalLength, principalPoint, key_point);
     double res_angle = robotOrientation + angle;
     Edge ray = createRay(robotPosition, res_angle); 
     std::pair<int, Point2f> temp = myMap.getNearestIntersecton(&ray);  
    // myMap.drawLine(&robotPosition, &temp.second, Scalar(255, 0, 0));
     if (temp.second.x != -1 && temp.second.y != -1) {
       //draw points
         if (Edge(&temp.second, &robotPosition).getLength() > 800)
             continue;
       Scalar color = Scalar((rand()%(255)), (rand()%(255)), (rand()%(255)));
       std::stringstream ss;ss << good_matches[i].queryIdx;         
       putText(ress, ss.str(), Point2f(newKeypoints[good_matches[i].queryIdx].pt.x+10, newKeypoints[good_matches[i].queryIdx].pt.y+10), 
        CV_FONT_HERSHEY_COMPLEX_SMALL, 0.6, color, 1, CV_AA);
       circle(ress, Point2f(newKeypoints[good_matches[i].queryIdx].pt.x, newKeypoints[good_matches[i].queryIdx].pt.y), 2, color, 3, 8);
       myMap.drawPoint(temp.second, 2, color);
       imshow("img", ress);
       imshow("map", myMap.getImage());
       cvWaitKey(1000);
       myMap.updateKeyPointPosition(temp.first, &temp.second, descr, robotPosition);
     }
  }
  previousImage = cvCloneImage(frameImage);
  return pair<double, int>(-1, -1);
}
bool isGoodIntersection(Point2f p1, Point2f p2, Point2f intersection, float radius) {
    Point2f center = Point2f((p1.x+p2.x)/2, (p1.y+p2.y)/2);
    return Edge(&center, &intersection).getLength()<radius;
}

/**
 * @param frameImage
 * undistort image and try to return robot position otherwise return point (-1, -1)
 */
Point2f localize(IplImage* frameImage) {
  myMap.drawMap();
  IplImage *t = cvCloneImage(frameImage);
  cvRemap(frameImage, t, mapx, mapy);  
  Mat frame = Mat(frameImage, false);
  vector<PairOfPoints> positions = matchWithMap(&frame);
  if (positions.size()==0) {
      return Point2f(-1, -1);
  } 
  vector<Point2f> intersections;
  int a = rand()%(positions.size());
  int b = rand()%(positions.size());  
  float angle = getAngle(focalLength, principalPoint, positions.at(a).first, positions.at(b).first);
  pair<Point2f, float> cc1 = computeCenter(&positions.at(a).second, &positions.at(b).second, angle);
  int prevA, prevB;
  for (int i=0; i<positions.size();i++) {
      a = rand()%(positions.size());
      b = rand()%(positions.size());
      prevA=a;prevB=b;
      if ((a == prevA &&b==prevB) ||(a==prevB && b==prevA))
          continue;
      angle = getAngle(focalLength, principalPoint, positions.at(a).first, positions.at(b).first);
      pair<Point2f, float> cc2 = computeCenter(&positions.at(a).second, &positions.at(b).second, angle);
      pair<Point2f, Point2f> intersection = circleIntersection(cc1.first, cc1.second, cc2.first, cc2.second);
      //check if isGoodIntersection
      if (isGoodIntersection(positions.at(a).second, positions.at(b).second, intersection.first, cc1.second)) {
        intersections.push_back(intersection.second);
        myMap.drawPoint(intersection.second, 2, Scalar(0,0,255));
      }
     if (isGoodIntersection(positions.at(a).second, positions.at(b).second, intersection.second, cc1.second)) {
        intersections.push_back(intersection.first);        
      myMap.drawPoint(intersection.first, 2, Scalar(0,0,255));
     }
  }
  imshow("localization", myMap.getImage());
  return Point2f(-1, -1);
}

/**
 * @param folderName
 * @param fileName
 * read log and count odometry error
 * draw robot positions into map
 */
void readLogForMapping(char *folderName, char* fileName) {
  FILE *f;
  char str[80];
  strcpy(str, folderName);
  strcat(str, fileName);
  f = fopen(str, "r");
  char line[256];
  char *pos;
  char frames[1000][20];
  int lOtack, pOtack;
  float kompas, orientation;
  int startX, startY, endX, endY;
  Point2f startPosition, endPosition;
  std::vector< pair<int, int> > odometria;
  fgets(line, 255, f);
  sscanf(line, "%f", &orientation);
  fgets(line, 255, f);
  sscanf(line, "%d %d", &startX, &startY);
  startPosition = Point2f(startX, startY);
  fgets(line, 255, f);
  sscanf(line, "%d %d", &endX, &endY);
  endPosition = Point2f(endX, endY);
  fgets(line, 255, f); //title log line
  int i=0;
  while (fgets(line, 255, f) != NULL) {
      pos = index(line, ' ');
      *pos = 0;
      strncpy(frames[i], line, 19);
      i++;
      pos++;
      sscanf(pos,"%*d %d %d %*d %*d %*d %*d %*d %*d %*d %*d %*d %f", &lOtack, &pOtack, &kompas);
      odometria.push_back(pair<int, int>(lOtack, pOtack));
      cout << line <<" "<< lOtack <<" "<< pOtack <<" "<< kompas <<"\n";
  }
  int start = round((odometria.at(0).first + odometria.at(0).second)/2);
  int end = round((odometria.at(odometria.size()-1).first + odometria.at(odometria.size()-1).second)/2);
  float scale = Edge(&endPosition, &startPosition).getLength() / (end - start);
  for (int i = 0; i<odometria.size();i++) {
      float posun = scale * round((odometria.at(i).first-odometria.at(0).first + odometria.at(i).second-odometria.at(0).second)/2);
      myMap.drawPoint(Point2f(startPosition.x, startPosition.y+posun), 2, Scalar(255, 0, 0));
      strcpy(str, folderName);
      char* imName = strcat(str, frames[i]);
      IplImage* img = cvLoadImage(imName, CV_LOAD_IMAGE_GRAYSCALE);      
      if (orientation>=45 && orientation<135)
        findInMap(img, Point2f(startPosition.x, startPosition.y+posun), degToRad(orientation));
      if (orientation>=135 && orientation<225)
        findInMap(img, Point2f(startPosition.x, startPosition.y-posun), degToRad(orientation));
      if (orientation>=225 && orientation<315)
        findInMap(img, Point2f(startPosition.x+posun, startPosition.y), degToRad(orientation));
      if (orientation>=315 || orientation<45)
        findInMap(img, Point2f(startPosition.x-posun, startPosition.y), degToRad(orientation));
  }
  fclose(f);
  imshow("Map Map", myMap.getImage());
  
}


void localizeOnMap() {
  myMap.readMapDescriptors("map.txt"); 
  imshow("MAP", myMap.getImage());
  IplImage* img_2 = cvLoadImage("rucneMapovanie/frame_1.jpg", CV_LOAD_IMAGE_GRAYSCALE);
  newKeypoints.clear();
  localize(img_2);
}
/*
 * read log and draw map
 */
void mapping() {//(Point robotPosition, char* logFileName)
  myMap.drawMap();
  readLogForMapping("rucneMapovanie/", "1400932920.log");
  myMap.drawMap();
  myMap.drawMapKeyPoints();
  imshow("map map",myMap.getImage());
//  double robotOrientation = 0;//180;  
//  Point2f startPosition = Point2f(4493, 36);//4478
//  float posun = 15*1.3;
//  //Mat img_1 = imread("rucneMapovanie/frame_0.jpg", CV_LOAD_IMAGE_GRAYSCALE);
//  //findInMap(&img_1, Point2f(startPosition.x, startPosition.y), (1.0/2.0*M_PI + degToRad(-2)));
//  IplImage *img_2 = cvLoadImage("rucneMapovanie/frame_1.jpg", CV_LOAD_IMAGE_GRAYSCALE);
//  init(img_2);
//  findInMap(img_2, Point2f(startPosition.x, startPosition.y+posun), (1.0/2.0*M_PI+ degToRad(-1)));
//  posun = 47*1.3; 
//  IplImage *img_3 = cvLoadImage("rucneMapovanie/frame_2.jpg", CV_LOAD_IMAGE_GRAYSCALE);
//  findInMap(img_3, Point(startPosition.x, startPosition.y+posun), (1.0/2.0*M_PI+ degToRad(-1)));
//  
  myMap.saveMap("map.txt");
  //myMap.drawMapKeyPoints();
//  imshow("map map",myMap.getImage());
}

#endif	/* SURFNAV_H */

