#include <stdio.h>
#include <iostream>
#include <opencv2/calib3d/calib3d.hpp>
#include "opencv2/core/core.hpp"
#include "opencv2/features2d/features2d.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/nonfree/features2d.hpp"

#include <curl/curl.h>
#include <cstdlib>
#include "Map.h"

  CURL *curl;
  CURLcode res;
  char* url = "http://www.patricia.poriadne.sk/SmelyZajko/index.php?r=site/map";
  char* taskUrl = "http://www.patricia.poriadne.sk/SmelyZajko/index.php?r=site/post";
  bool isWorking = false;
  
  using namespace cv;
/**
 * convert int to string
 */  
std::string convertInt(int number) {
   std::stringstream ss;
   ss << number;
   return ss.str();
}

/**
 * @param urlAddress url where data will be processed
 * @param param $_POST params
 * make post to server
 */
CURLcode post(char* urlAddress, char* param) {
  curl_easy_setopt(curl, CURLOPT_URL, urlAddress);
  curl_easy_setopt(curl, CURLOPT_POSTFIELDS, param);
  res = curl_easy_perform(curl);
  if (res != CURLE_OK) {
    printf("POST failed: %s\n", curl_easy_strerror(res));  
  }
  return res;
}

/**
 * undistort image after calibration
 * data for calibration are loaded from Intrinsics.xml, Distortion.xml
 */
void undistortImage() {
 IplImage *image;

 CvMat *intrinsic = (CvMat*)cvLoad("Intrinsics.xml");
 CvMat *distortion = (CvMat*)cvLoad("Distortion.xml");
 IplImage* mapx = cvCreateImage(cvGetSize(image), IPL_DEPTH_32F, 1);
 IplImage* mapy = cvCreateImage(cvGetSize(image), IPL_DEPTH_32F, 1);
 cvInitUndistortMap(intrinsic,distortion,mapx,mapy);

 // show undistord
 while (image) {
  IplImage *t = cvCloneImage(image);
  cvRemap( t, image, mapx, mapy);  // Undistort image
  cvReleaseImage(&t);
  cvShowImage("Undistort", image);  // Show corrected image
 }
}

void surf(Map* _map);
void mapping();
void init();
void calibrate();
void localizeOnMap();

void serverCommunication() {
 //init curl
  curl_global_init(CURL_GLOBAL_ALL);
  curl = curl_easy_init();
  isWorking = true;
  if (curl) {
      if (!isWorking) {
      //get new task with format <from_roomIndex"," to_roomIndex"," time_to_be_delivered>
        post(taskUrl, "getTask=true");
        curl_easy_cleanup(curl);
      } else {
      //set new position and delivery status
        post(url, "position=I 7&taskDone=true");
        curl_easy_cleanup(curl);
      }
  }
  curl_global_cleanup();
}

int main() {
    Map myMap = Map();
    myMap.readMap("plan.svg");

    float robotWidth = 50.4;
    init();
    waitKey(100);
    mapping();
    localizeOnMap();
    waitKey(0);
  return 0;
}