<?php

/**
 * This is the model class for table "path_log".
 *
 * The followings are the available columns in table 'path_log':
 * @property integer $lid
 * @property integer $from
 * @property integer $to
 * @property string $time
 * @property integer $state
 *
 * The followings are the available model relations:
 * @property Rooms $to0
 * @property Rooms $from0
 */
class PathLog extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'path_log';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		return array(
			array('from, to, time', 'required'),
			array('from, to, state', 'numerical', 'integerOnly'=>true),
			array('lid, from, to, time, state', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		return array(
			'to0' => array(self::BELONGS_TO, 'Rooms', 'to'),
			'from0' => array(self::BELONGS_TO, 'Rooms', 'from'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
            return array(
                    'lid' => 'Id',
                    'from'=>'Štart',
                    'to' => 'Cieľ',
                    'time' => 'Čas doručenia',
            );
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		$criteria=new CDbCriteria;

		$criteria->compare('lid',$this->lid);
		$criteria->compare('from',$this->from);
		$criteria->compare('to',$this->to);
		$criteria->compare('time',$this->time,true);
		$criteria->compare('state',$this->state);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
        /**
	 * @return array of room_numbers (rid=>room_number)
	 */
        public function getRooms() 
        {
          return CHtml::listData(Rooms::model()->findAll(), 'rid', 'room_number');
        }
        /**
	 * @return array of delivery states
	 */
        public static function getStates() 
        {
          return array('0'=>'Plánované','1'=>'Doručujem','2'=>'Doručené', '3'=>'Neúspešné');
        }
        /**
	 * @return deliveryState
	 */
        public function getState($id) 
        {
          $states = array('0'=>'Plánované','1'=>'Doručujem','2'=>'Doručené', '3'=>'Neúspešné') ;
          return $states[$id];
        }
        
        public function getRoomName($id) 
        {
            $rooms = CHtml::listData(Rooms::model()->findAll(), 'rid', 'room_number');
            return $rooms[$id];
        }
        
        public function validateRooms() 
        {
            if ($this->from != $this->to)
              return true;
            return false;
        }

        public function solveActualTask() 
        {
          $tasks = PathLog::model()->findAllByAttributes(array('state'=>'1'));
          if ($tasks != null) {
              $tasks[0]->setAttribute('state', '2');
              $tasks[0]->save();
          }
        }
        
        public function setActualTaskUndelivered() 
        {
          $tasks = PathLog::model()->findAllByAttributes(array('state'=>'1'));
          if ($tasks != null) {
              $tasks[0]->setAttribute('state', '3');
              $tasks[0]->save();
          }
        }
        
	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return PathLog the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
